# Form implementation generated from reading ui file 'mainwindow.ui'
#
# Created by: PyQt6 UI code generator 6.8.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1257, 900)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon.fromTheme("emblem-shared")
        MainWindow.setWindowIcon(icon)
        self.centralwidget = QtWidgets.QWidget(parent=MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.formLayout = QtWidgets.QFormLayout(self.centralwidget)
        self.formLayout.setObjectName("formLayout")
        self.toolBox = QtWidgets.QToolBox(parent=self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolBox.sizePolicy().hasHeightForWidth())
        self.toolBox.setSizePolicy(sizePolicy)
        self.toolBox.setMinimumSize(QtCore.QSize(400, 0))
        self.toolBox.setObjectName("toolBox")
        self.pageSource = QtWidgets.QWidget()
        self.pageSource.setGeometry(QtCore.QRect(0, 0, 400, 824))
        self.pageSource.setObjectName("pageSource")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.pageSource)
        self.gridLayout_3.setObjectName("gridLayout_3")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_3.addItem(spacerItem, 3, 0, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(parent=self.pageSource)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.labelEegStatus = QtWidgets.QLabel(parent=self.groupBox)
        self.labelEegStatus.setEnabled(False)
        self.labelEegStatus.setObjectName("labelEegStatus")
        self.gridLayout_4.addWidget(self.labelEegStatus, 5, 1, 1, 1)
        self.comboBoxStreamName01 = QtWidgets.QComboBox(parent=self.groupBox)
        self.comboBoxStreamName01.setObjectName("comboBoxStreamName01")
        self.gridLayout_4.addWidget(self.comboBoxStreamName01, 3, 1, 1, 1)
        self.labelLslStatus = QtWidgets.QLabel(parent=self.groupBox)
        self.labelLslStatus.setEnabled(False)
        self.labelLslStatus.setObjectName("labelLslStatus")
        self.gridLayout_4.addWidget(self.labelLslStatus, 4, 1, 1, 1)
        self.checkBoxConnect01 = QtWidgets.QCheckBox(parent=self.groupBox)
        self.checkBoxConnect01.setObjectName("checkBoxConnect01")
        self.gridLayout_4.addWidget(self.checkBoxConnect01, 3, 0, 1, 1)
        self.lineEditName01 = QtWidgets.QLineEdit(parent=self.groupBox)
        self.lineEditName01.setText("")
        self.lineEditName01.setObjectName("lineEditName01")
        self.gridLayout_4.addWidget(self.lineEditName01, 0, 1, 1, 1)
        self.labelFrequencyBands = QtWidgets.QLabel(parent=self.groupBox)
        self.labelFrequencyBands.setEnabled(False)
        self.labelFrequencyBands.setObjectName("labelFrequencyBands")
        self.gridLayout_4.addWidget(self.labelFrequencyBands, 6, 1, 1, 1)
        self.gridLayout_3.addWidget(self.groupBox, 0, 0, 1, 1)
        self.groupBoxVisualisation = QtWidgets.QGroupBox(parent=self.pageSource)
        self.groupBoxVisualisation.setObjectName("groupBoxVisualisation")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBoxVisualisation)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.checkBoxDspEegPsd = QtWidgets.QCheckBox(parent=self.groupBoxVisualisation)
        self.checkBoxDspEegPsd.setEnabled(False)
        self.checkBoxDspEegPsd.setChecked(False)
        self.checkBoxDspEegPsd.setObjectName("checkBoxDspEegPsd")
        self.gridLayout_6.addWidget(self.checkBoxDspEegPsd, 0, 0, 1, 1)
        self.checkBoxDspDancingDots = QtWidgets.QCheckBox(parent=self.groupBoxVisualisation)
        self.checkBoxDspDancingDots.setEnabled(False)
        self.checkBoxDspDancingDots.setObjectName("checkBoxDspDancingDots")
        self.gridLayout_6.addWidget(self.checkBoxDspDancingDots, 3, 0, 1, 1)
        self.gridLayout_3.addWidget(self.groupBoxVisualisation, 2, 0, 1, 1)
        self.toolBox.addItem(self.pageSource, "")
        self.pageSettings = QtWidgets.QWidget()
        self.pageSettings.setGeometry(QtCore.QRect(0, 0, 400, 824))
        self.pageSettings.setObjectName("pageSettings")
        self.gridLayout_31 = QtWidgets.QGridLayout(self.pageSettings)
        self.gridLayout_31.setObjectName("gridLayout_31")
        self.label = QtWidgets.QLabel(parent=self.pageSettings)
        self.label.setObjectName("label")
        self.gridLayout_31.addWidget(self.label, 0, 0, 1, 1)
        self.toolBox.addItem(self.pageSettings, "")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.toolBox)
        self.tabWidget = QtWidgets.QTabWidget(parent=self.centralwidget)
        self.tabWidget.setObjectName("tabWidget")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.tabWidget)
        MainWindow.setCentralWidget(self.centralwidget)
        self.actionQuit = QtGui.QAction(parent=MainWindow)
        self.actionQuit.setObjectName("actionQuit")

        self.retranslateUi(MainWindow)
        self.toolBox.setCurrentIndex(0)
        self.tabWidget.setCurrentIndex(-1)
        self.actionQuit.triggered.connect(MainWindow.close) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Samadhi"))
        self.groupBox.setTitle(_translate("MainWindow", "Data Source"))
        self.labelEegStatus.setText(_translate("MainWindow", "EEG Status"))
        self.labelLslStatus.setText(_translate("MainWindow", "LSL Status"))
        self.checkBoxConnect01.setText(_translate("MainWindow", "Click to connect"))
        self.lineEditName01.setPlaceholderText(_translate("MainWindow", "Name of Person"))
        self.labelFrequencyBands.setText(_translate("MainWindow", "Frequency Bands"))
        self.groupBoxVisualisation.setTitle(_translate("MainWindow", "Visualisation"))
        self.checkBoxDspEegPsd.setText(_translate("MainWindow", "EEG and Spectrum"))
        self.checkBoxDspDancingDots.setText(_translate("MainWindow", "Dancing Dots"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.pageSource), _translate("MainWindow", "Data Source"))
        self.label.setText(_translate("MainWindow", "<html><head/><body><p>Samadhi EEG<br/><br/>Version 2025.2<br/><br/>Software to explore, test, and make available<br/>different visualisations for EEG.<br/><br/>License: GPLv4<br/><br/>Web: <a href=\"https://samadhi-eeg.info\"><span style=\" text-decoration: underline; color:#1d99f3;\">https://samadhi-eeg.info<br/></span></a>Source code, bugs, issues: <a href=\"https://github.com/jkanev/samadhi\"><span style=\" text-decoration: underline; color:#1d99f3;\">github.com/jkanev/samadhi</span></a></p></body></html>"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.pageSettings), _translate("MainWindow", "About"))
        self.actionQuit.setText(_translate("MainWindow", "Quit"))
